<?php

namespace Drupal\entity_reports\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\entity_reports\ReportGenerator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class EntityReportsController handle routes for entity reports.
 */
class EntityReportsController extends ControllerBase {

  /**
   * The report generator service.
   *
   * @var \Drupal\entity_reports\ReportGenerator
   */
  protected $generator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_reports.generator')
    );
  }

  /**
   * EntityReportsController constructor.
   *
   * @param \Drupal\entity_reports\ReportGenerator $reportGenerator
   *   The report generator service.
   */
  public function __construct(ReportGenerator $reportGenerator) {
    $this->generator = $reportGenerator;
  }

  /**
   * Returns the title for an Entity report page.
   *
   * @param string $entity_type
   *   Entity type, i.e. node, taxonomy_term.
   *
   * @return string
   *   The report page title.
   */
  public function displayReportTitle($entity_type) {
    return $this->t('Entity reports: @entity_type_label', [
      '@entity_type_label' => $this->entityTypeManager()->getDefinition($entity_type)->getLabel(),
    ]);
  }

  /**
   * Displays a report for a given entity type.
   *
   * @param string $entity_type
   *   Entity type, i.e. node, taxonomy_term.
   *
   * @return array
   *   A renderable array as expected by the renderer service.
   */
  public function displayReport($entity_type) {
    $definition = $this->entityTypeManager()->getDefinition($entity_type);

    $build = [];

    $structure = $this->generator->getEntityTypeStructure($entity_type);
    foreach ($structure as $bundle => $data) {
      $rows = [];
      foreach ($data['fields'] as $field) {
        $rows[] = [
          'label' => $field['label'],
          'machine_name' => $field['machine_name'],
          'description' => $field['description'],
          'type' => $field['type'],
          'required' => $field['required_human'],
          'translatable' => $field['translatable_human'],
          'target' => $field['target'],
          'cardinality' => $field['cardinality_human'],
        ];
      }
      $build['info'] = [
        '#markup' => $this->t('Open each @bundle_label below to see details about its fields. You can also download as <a href=":json">JSON</a> or <a href=":xml">XML</a>.', [
          '@bundle_label' => $definition->getBundleLabel(),
          ':json' => Url::fromRoute("entity_reports.entity_structure.$entity_type.json")->toString(),
          ':xml' => Url::fromRoute("entity_reports.entity_structure.$entity_type.xml")->toString(),
        ]),
      ];
      $build[$bundle . '_wrapper'] = [
        '#title' => $data['label'],
        '#type' => 'details',
        '#open' => FALSE,
        'fields' => [
          '#type' => 'table',
          '#header' => [
            $this->t('Field name'),
            $this->t('Machine name'),
            $this->t('Description'),
            $this->t('Data type'),
            $this->t('Required'),
            $this->t('Translatable'),
            $this->t('Target'),
            $this->t('Cardinality'),
          ],
          '#rows' => $rows,
          '#empty' => $this->t('No fields configured.'),
        ],
      ];
    }

    return $build;
  }

  /**
   * Lists available entity type reports.
   *
   * @return array
   *   A renderable array as expected by the renderer service.
   */
  public function availableEntityTypes() {
    $build = [];

    $items = [];
    $configured_types = $this->config('entity_reports.settings')->get('reported_entity_types');
    foreach ($this->entityTypeManager()->getDefinitions() as $entity_type_id => $entity_type) {
      if ($entity_type->entityClassImplements(FieldableEntityInterface::class)) {
        if (!empty($configured_types) && !in_array($entity_type_id, $configured_types, TRUE)) {
          continue;
        }
        $url = Url::fromRoute("entity_reports.entity_structure.$entity_type_id");
        $items[(string) $entity_type->getLabel()] = Link::fromTextAndUrl($entity_type->getLabel(), $url)->toString();
      }
    }
    ksort($items);
    $build['content'] = [
      '#title' => $this->t('Available entity reports:'),
      '#theme' => 'item_list',
      '#items' => $items,
    ];

    return $build;
  }

  /**
   * Displays a report for a given entity type.
   *
   * @param string $entity_type
   *   Entity type, i.e. node, taxonomy_term.
   * @param string $export_format
   *   The export format, i.e. json, xml.
   */
  public function exportReport($entity_type, $export_format) {
    $structure = $this->generator->getEntityTypeStructure($entity_type);
    return $this->export(['bundles' => $structure], $export_format);
  }

  /**
   * Provides the content in a specified format.
   *
   * @param array $content
   *   The content of response.
   * @param string $type
   *   The content type of response.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object.
   */
  public function export(array $content, $type = 'json') {
    $response = new Response();
    $response->headers->set('Content-Type', 'application/' . $type);
    switch ($type) {
      case 'xml':
        if (count($content) == 1) {
          $rootElement = key($content);
          $content = reset($content);
        }
        else {
          $rootElement = 'root';
        }
        $xml = new \SimpleXMLElement("<{$rootElement}/>");
        $this->arrayToXml($content, $xml, $rootElement);
        $content = $xml->asXML();
        break;

      default:
        $content = json_encode($content);
    }
    $response->setContent($content);
    return $response;
  }

  /**
   * Adds an array to an XML element.
   *
   * @param array $array
   *   The input array.
   * @param \SimpleXMLElement $xml
   *   The output XML element.
   * @param string $parentKey
   *   The root element name.
   */
  protected function arrayToXml(array $array, \SimpleXMLElement &$xml, $parentKey) {
    foreach ($array as $key => $value) {
      $xmlKey = !is_numeric($key) ? $key : "item$key";
      $attributes = [];
      switch ($parentKey) {
        case 'contentTypes':
          $xmlKey = 'contentType';
          $attributes['id'] = $key;
          break;

        case 'vocabularies':
          $xmlKey = 'vocabulary';
          $attributes['id'] = $key;
          break;

        case 'fields':
          $xmlKey = 'field';
          $attributes['id'] = $key;
          break;

        case 'terms':
          $xmlKey = 'term';
          $attributes['id'] = $key;
          break;
      }
      if (is_array($value)) {
        $subnode = $xml->addChild($xmlKey);
        $this->arrayToXml($value, $subnode, $xmlKey);
      }
      else {
        $subnode = $xml->addChild($xmlKey, $value);
      }
      foreach ($attributes as $attrKey => $attrValue) {
        $subnode->addAttribute($attrKey, $attrValue);
      }
    }
  }

}
